from django import forms
from dal.autocomplete import ModelSelect2
from django_geo_db.models import UserLocation, City, Location, GeoCoordinate
from django_geo_db.widgets import GeocoordinateWidget


class GeocoordinateForm(forms.ModelForm):
    class Meta:
        model = GeoCoordinate
        fields = [
            'generated_name',
            'lat',
            'lon',
        ]
        widgets = {
            'generated_name': GeocoordinateWidget
        }


class UserLocationForm(forms.ModelForm):
    class Meta:
        model = UserLocation
        widgets = {
            'location': ModelSelect2(url='location-autocomplete')
        }
        fields = '__all__'



class LocationForm(forms.ModelForm):
    class Meta:
        model = Location
        widgets = {
            'zipcode': ModelSelect2(url='zipcode-autocomplete'),
            'city': ModelSelect2(url='city-autocomplete'),
            'geocoordinate': ModelSelect2(url='geocoordinate-autocomplete'),
            }
        fields = '__all__'




class CityForm(forms.ModelForm):
    class Meta:
        model = City
        widgets = {
            'zipcode': ModelSelect2(url='zipcode-autocomplete'),
            }
        fields = '__all__'




from django.db.models import Q
from django.shortcuts import Http404
from dal import autocomplete

from rest_framework import generics, permissions, mixins, filters
from rest_framework.decorators import api_view, APIView
from rest_framework.response import Response
from django_geo_db import serializers
from django_geo_db.serializers import LocationSerializer
from django_geo_db.services import GEO_DAL
from django_geo_db.models import Continent, Country, State, Location, City, Zipcode, GeoCoordinate, UserLocation


class UsersLocationAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        if not self.request.user.is_authenticated():
            return Location.objects.none()

        user = self.request.user
        qs = UserLocation.objects.filter(user=user)

        if self.q:
            qs = qs.filter(
                Q(location__generated_name__icontains=self.q),
            )
        return qs


class NamedLocationAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        if not self.request.user.is_authenticated():
            return Location.objects.none()

        qs = Location.objects.filter(name__isnull=False)

        if self.q:
            qs = qs.filter(
                Q(generated_name__istartswith=self.q)
            )
        return qs


class LocationAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        if not self.request.user.is_authenticated():
            return Location.objects.none()

        qs = Location.objects.all()

        if self.q:
            qs = qs.filter(
                Q(generated_name__istartswith=self.q) |
                Q(generated_name__endswith=self.q)
            )
        return qs


class PublicLocationsAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):

        qs = Location.objects.public_locations()

        if self.q:
            qs = qs.filter(
                Q(generated_name__contains=self.q)
            )
        return qs


class CityAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        if not self.request.user.is_authenticated():
            return City.objects.none()

        qs = City.objects.all()

        if self.q:
            qs = qs.filter(
                Q(generated_name__istartswith=self.q) |
                Q(generated_name__endswith=self.q)
            )
        return qs


class ZipcodeAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        qs = Zipcode.objects.all()

        if self.q:
            qs = qs.filter(
                Q(generated_name__iendswith=self.q)
            )
        return qs


class GeoCoordinateAutocomplete(autocomplete.Select2QuerySetView):
    def get_queryset(self):
        if not self.request.user.is_authenticated():
            return GeoCoordinate.objects.none()

        qs = GeoCoordinate.objects.all()

        if self.q:
            qs = qs.filter(
                Q(generated_name__istartswith=self.q)
            )
        return qs




from django.db import models
from django_geo_db.utilities import get_lat_lon_from_string

class GeoCoordinateManager(models.Manager):

    def get_or_create_by_lat_lon(self, lat, lon):
        lat, lon = get_lat_lon_from_string('{0} {1}'.format(lat, lon))
        return super(GeoCoordinateManager, self).get_or_create(lat=lat, lon=lon)


class LocationManager(models.Manager):

    def public_locations(self):
        return super(LocationManager, self).filter(name__isnull=False, is_private=False)



from datetime import datetime
from django.core.management.base import BaseCommand
from django_geo_db.models import City, Continent, Country, \
    GeoCoordinate, State, Location, Zipcode
from django_geo_db.services import generate_current_us_cities_list, generate_current_us_states_list, generate_countries
from django_geo_db.utilities import get_standardized_coordinate


def get_minutes(timeDelta):
    seconds = timeDelta.total_seconds()
    minutes = (seconds % 3600) // 60
    return minutes


class Command(BaseCommand):
    help = "Setups all Geographical information for US based cities."


    def __get_geocoordinate(self, lat, lon):
        lat = get_standardized_coordinate(lat)
        lon = get_standardized_coordinate(lon)
        try:
            geoCoord = GeoCoordinate.objects.create(lat=lat, lon=lon)
        except:
            try:
                geoCoord = GeoCoordinate.objects.get(lat=lat, lon=lon)
            except Exception as e:
                print('Lat: {0}, Lon: {1}'.format(lat, lon))
                raise e
        return geoCoord



    def handle(self, *args, **options):
        totalStart = datetime.now()

        print('Populating Continents and Countries')
        start = datetime.now()
        self.create_all_countries_and_continents()
        end = datetime.now()
        print('Continents and Countries took {0} seconds.'.format((end - start).total_seconds()))

        print('Populating US States')
        start = datetime.now()
        self.create_all_us_states()
        end = datetime.now()
        print('US States took {0} seconds.'.format((end - start).total_seconds()))

        print('Populating US Cities and Zips')
        start = datetime.now()
        self.create_all_cities_and_zipcodes()
        end = datetime.now()
        print('US Cities and Zips took {0} minutes.'.format(get_minutes(end - start)))

        totalEnd = datetime.now()
        print('Total import took {0} minutes.'.format(get_minutes(totalEnd - totalStart)))


    def create_all_countries_and_continents(self):
        continents = {}
        count = 0
        for country, continent, abbrev, lat, lon in generate_countries():
            try:
                if continent not in continents:
                    continentObj = Continent.objects.create(name=continent)
                    continents[continent] = continentObj
                geocoordinate = self.__get_geocoordinate(lat, lon)
                countryObj = Country.objects.create(continent=continents[continent], name=country, geocoordinate=geocoordinate, abbreviation=abbrev)
                Location.objects.create(country=countryObj)
                count += 1
            except:
                raise Exception('Exception occurred processing {0} country.'.format(country))



    def create_all_us_states(self):
        locationList = []
        usCountry = Country.objects.get(name='United States of America')
        for state, abbreviation, latitude, longitude in generate_current_us_states_list():
            try:
                geoCoordinate = self.__get_geocoordinate(latitude, longitude)
                stateObj = State(country=usCountry, name=state, abbreviation=abbreviation, geocoordinate=geoCoordinate)
                stateObj.save()
                locationObj = Location(country=usCountry, state=stateObj)
                locationList.append(locationObj)
            except:
                raise Exception('Exception occurred processing {0} state.'.format(state))
        Location.objects.bulk_create(locationList)


    def create_all_cities_and_zipcodes(self):
        count = State.objects.all().count()
        stateDict = {}
        usCountry = Country.objects.get(name='United States of America')
        for state in State.objects.filter(country=usCountry):
            stateDict[state.abbreviation] = state
        uniqueCities = set()
        noLat = []
        for zip, lat, lon, city, state, timezone in generate_current_us_cities_list():
            if not lat:
                noLat.append((zip, lat, lon, city, state, timezone))
                continue
            stateObj = stateDict[state]
            cityObj = None
            try:
                geoCoordinate = self.__get_geocoordinate(lat, lon)
                keyFound = False
                key = '{0}-{1}'.format(state, city)
                if key not in uniqueCities:
                    uniqueCities.add(key)
                    cityObj = City.objects.create(state=stateObj, name=city, geocoordinate=geoCoordinate)
                    Location.objects.create(country=usCountry, state=stateObj, city=cityObj)
                    keyFound = False
                if not cityObj:
                    cityObj = City.objects.get(state=stateObj, name=city)
                    keyFound = True
                zipObj = Zipcode.objects.create(city=cityObj, zipcode=zip, geocoordinate=geoCoordinate, timezone=timezone)
                if stateObj != cityObj.state:
                    raise Exception('{0} is not {1} in city {2}. KeyFound {3}. Key {4}'.format(stateObj, cityObj.state, cityObj, keyFound, key))
                Location.objects.create(country=usCountry, state=stateObj, city=cityObj, zipcode=zipObj)
            except:
                print('zip: ' + zip)
                print('StateObj: ' + str(stateObj))
                print('State: ' + str(state))
                print('Key: ' + str(key))
                raise Exception('Exception occurred while processing Zipcode {0}'.format(zip))
        for zip, lat, lon, city, state, timezone in noLat:
            key = '{0}-{1}'.format(state, city)
            stateObj = stateDict[state]
            cityObj = City.objects.filter(state=stateObj, name=city).first()
            if not cityObj:
                cityObj = City.objects.filter(state=stateObj).first()
            zipObj = Zipcode.objects.create(city=cityObj, zipcode=zip, geocoordinate=cityObj.geocoordinate, timezone=timezone)
            Location.objects.create(country=usCountry, state=stateObj, city=cityObj, zipcode=zipObj)







from django.db.models import Q
from django.shortcuts import Http404

from rest_framework import generics, permissions, mixins, filters
from rest_framework.decorators import api_view, APIView
from rest_framework.response import Response
from django_geo_db import serializers
from django_geo_db.serializers import LocationSerializer
from django_geo_db.services import GEO_DAL
from django_geo_db.models import Continent, Country, State, Location, City, Zipcode, GeoCoordinate, UserLocation


class LocationDetail(APIView):
    def get_object(self, pk):
        try:
            return Location.objects.get(pk=pk)
        except Location.DoesNotExist:
            raise Http404

    def get(self, request, pk, format=None):
        snippet = self.get_object(pk)
        serializer = LocationSerializer(snippet)
        return Response(serializer.data)


class LocationList(generics.ListAPIView):
    serializer_class = LocationSerializer
    permission_classes = (permissions.IsAuthenticatedOrReadOnly,)


    def get_queryset(self):
        return GEO_DAL.get_all_named_locations()


class ContinentList(mixins.ListModelMixin,
                  generics.GenericAPIView):
    queryset = Continent.objects.all()
    serializer_class = serializers.ContinentSerializer
    filter_fields =  ('name', )

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class ContinentDetails(mixins.RetrieveModelMixin,
                       generics.GenericAPIView):
    queryset = Continent.objects.all()
    serializer_class = serializers.ContinentSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class CountryList(mixins.ListModelMixin,
                    generics.GenericAPIView):
    queryset = Country.objects.all()
    serializer_class = serializers.CountrySerializer
    filter_fields =  ('continent__continent_id', 'name', 'abbreviation')

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class CountryDetails(mixins.RetrieveModelMixin,
                       generics.GenericAPIView):
    queryset = Country.objects.all()
    serializer_class = serializers.CountrySerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class StateList(mixins.ListModelMixin,
                generics.GenericAPIView):
    queryset = State.objects.all()
    serializer_class = serializers.StateSerializer
    filter_fields =  ('country__country_id', 'name')

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class StateDetails(mixins.RetrieveModelMixin,
                     generics.GenericAPIView):
    queryset = State.objects.all()
    serializer_class = serializers.StateSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class CityList(mixins.ListModelMixin,
                generics.GenericAPIView):
    queryset = City.objects.all()
    serializer_class = serializers.CitySerializer
    filter_fields =  ('state__state_id', 'name')

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class CityDetails(mixins.RetrieveModelMixin,
                   generics.GenericAPIView):
    queryset = City.objects.all()
    serializer_class = serializers.CitySerializer
    filter_fields =  ('country__country_id', 'name', '')

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class ZipcodeList(mixins.ListModelMixin,
               generics.GenericAPIView):
    queryset = Zipcode.objects.all()
    serializer_class = serializers.ZipcodeSerializer
    filter_fields =  ('zipcode', 'city__city_id')


    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class ZipcodeDetails(mixins.RetrieveModelMixin,
                  generics.GenericAPIView):
    queryset = Zipcode.objects.all()
    serializer_class = serializers.ZipcodeSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)


class GeoCoordinateList(mixins.ListModelMixin,
                  generics.GenericAPIView):
    queryset = GeoCoordinate.objects.all()
    serializer_class = serializers.GeoCoordinateSerializer

    def get(self, request, *args, **kwargs):
        return self.list(request, *args, **kwargs)


class GeoCoordinateDetails(mixins.RetrieveModelMixin,
                     generics.GenericAPIView):
    queryset = GeoCoordinate.objects.all()
    serializer_class = serializers.GeoCoordinateSerializer

    def get(self, request, *args, **kwargs):
        return self.retrieve(request, *args, **kwargs)





class GoogleMapsSettings:
    lat = None
    lon = None
    zoom = None

    def __init__(self, *args, **kwargs):
        for k in kwargs.keys():
            if k in ['lat', 'lon', 'zoom']:
                self.__setattr__(k, kwargs[k])



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-16 20:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_geo_db', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='location',
            name='is_private',
            field=models.BooleanField(default=False, help_text='Is this a location that should not be available to be seen publically?'),
        ),
    ]



import os
import csv
from django_geo_db.models import UserLocation, Zipcode, Location

US_CITIES_FILE = 'us-cities-and-zips.csv'
US_STATES_FILE = 'us-states.csv'
COUNTRIES_FILE = 'countries.csv'

class GeographyDAL:

    def get_all_named_locations(self, include_private=False):
        objects = Location.objects.filter(name__isnull=False)
        if not include_private:
            objects = objects.filter(is_private=False)
        return objects

    def get_location_by_id(self, id):
        return Location.objects.get(pk=id)

    def get_users_locations(self, user):
        return UserLocation.objects.filter(user=user).values_list('location', flat=True)

    def append_user_location(self, user, locationUsedByUser):
        obj, created = UserLocation.objects.get_or_create(user=user, location=locationUsedByUser)
        if not created:
            obj.save()  # This triggers the updating of the date.

    def get_zipcode_by_zip(self, zipcode):
        return Zipcode.objects.get(zipcode=zipcode)

    def create_location(self, zipcode, coordinate, name):
        city = zipcode.city
        state = city.state
        country = state.country
        location, created = Location.objects.get_or_create(country=country, state=state, city=city, zipcode=zipcode, geocoordinate=coordinate, name=name)
        return location

    def geocode_zipcode_from_lat_lon(self, lat, lon):
        """
        Only use this method if you have googlemaps installed.
        """
        import googlemaps
        from django.conf import settings
        gmaps = googlemaps.Client(key=settings.GOOGLE_MAPS_API_KEY)
        reverse_geocode_result = gmaps.reverse_geocode((40.714224, -73.961452))
        zipcodeObj = None
        for index in range(0, len(reverse_geocode_result[0]['address_components'])):
            obj = reverse_geocode_result[0]['address_components']
            if obj[index]['types'][0] == 'postal_code':
                zipcode = obj[index]['short_name']
                zipcodeObj = Zipcode.objects.get(zipcode=zipcode)
                break
        return zipcodeObj

GEO_DAL = GeographyDAL()


def generate_current_us_states_list():
    """
    Iterates through a list of all of the US States.
    (state, abbreviation, latitude, longitude)
    :return:
    """
    module_dir = os.path.dirname(__file__)  # get current directory
    file_path = os.path.join(module_dir, US_STATES_FILE)
    with open(file_path) as file:
        reader = csv.DictReader(file)
        for row in reader:
            state = row['state'].strip()
            abbreviation = row['abbreviation'].strip()
            latitude = row['latitude'].strip()
            longitude = row['longitude'].strip()
            yield (state, abbreviation, latitude, longitude)


def generate_current_us_cities_list():
    """
    Iterates through a list of all of the US cities.
    (zip_code,latitude,longitude,city,state, timezone)
    :return:
    """
    module_dir = os.path.dirname(__file__)  # get current directory
    file_path = os.path.join(module_dir, US_CITIES_FILE)
    with open(file_path) as file:
        reader = csv.DictReader(file)
        for row in reader:
            state = row['state'].strip()
            city = row['city'].strip()
            lat = row['latitude'].strip()
            lon = row['longitude'].strip()
            zip = row['zip'].strip()
            timezone = row['timezone']
            yield (zip, lat, lon, city, state, timezone)


def generate_countries():
    """
    Iterates through a list of all of the US cities.
    (country name, continent,abbreviation,latitude,longitude)
    :return:
    """
    module_dir = os.path.dirname(__file__)  # get current directory
    file_path = os.path.join(module_dir, COUNTRIES_FILE)
    with open(file_path) as file:
        reader = csv.DictReader(file)
        for row in reader:
            abbr = row['abbreviation'].strip()
            lat = row['latitude'].strip()
            lon = row['longitude'].strip()
            country = row['name'].strip().replace('"', '')
            continent = row['continent'].strip()
            yield (country, continent, abbr, lat, lon)



from rest_framework import serializers
from django_geo_db.models import Location, GeoCoordinate, Zipcode, Continent, Country, State, City


class GeoCoordinateSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField('geocoordinate-detail')
    class Meta:
        model = GeoCoordinate
        fields = ('geocoordinate_id', 'lat', 'lon', 'generated_name', 'url')


class ContinentSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField('continent-detail')
    class Meta:
        model = Continent
        fields = ('continent_id', 'name', 'url')


class CountrySerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField('country-detail')
    class Meta:
        model = Country
        fields = ('country_id', 'continent', 'name', 'abbreviation', 'geocoordinate', 'url')


class StateSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField('state-detail')
    class Meta:
        model = State
        fields = ('state_id', 'country', 'name', 'abbreviation', 'geocoordinate', 'generated_name', 'url')


class CitySerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField('city-detail')
    class Meta:
        model = City
        fields = ('city_id', 'state', 'name', 'geocoordinate', 'generated_name', 'url')


class ZipcodeSerializer(serializers.HyperlinkedModelSerializer):
    url = serializers.HyperlinkedIdentityField('zipcode-detail')
    class Meta:
        model = Zipcode
        fields = ('zipcode_id', 'city', 'zipcode', 'geocoordinate', 'timezone', 'url')


class LocationSerializer(serializers.ModelSerializer):
    lat = serializers.SerializerMethodField()
    lon = serializers.SerializerMethodField()
    url = serializers.HyperlinkedIdentityField('location-detail')

    def get_lat(self, obj):
        return str(obj.geocoordinate.lat)

    def get_lon(self, obj):
        return str(obj.geocoordinate.lon)

    class Meta:
        model = Location
        fields = ('location_id', 'country', 'city', 'zipcode', 'geocoordinate', 'lat', 'lon', 'name', 'generated_name', 'url')




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django_geo_db.models import IntegerRangeField
from django.conf import settings
import datetime


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='City',
            fields=[
                ('city_id', models.AutoField(serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('generated_name', models.CharField(null=True, max_length=50, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Continent',
            fields=[
                ('continent_id', models.AutoField(serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=50, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='Country',
            fields=[
                ('country_id', models.AutoField(serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('abbreviation', models.CharField(max_length=2, unique=True)),
                ('continent', models.ForeignKey(to='django_geo_db.Continent')),
            ],
        ),
        migrations.CreateModel(
            name='GeoCoordinate',
            fields=[
                ('geocoordinate_id', models.AutoField(serialize=False, primary_key=True)),
                ('lat', models.DecimalField(decimal_places=5, max_digits=7)),
                ('lon', models.DecimalField(decimal_places=5, max_digits=8)),
                ('generated_name', models.CharField(null=True, max_length=50, blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('location_id', models.AutoField(serialize=False, primary_key=True)),
                ('name', models.CharField(null=True, max_length=30, blank=True)),
                ('generated_name', models.CharField(null=True, max_length=50, blank=True)),
                ('city', models.ForeignKey(null=True, to='django_geo_db.City', blank=True)),
                ('country', models.ForeignKey(to='django_geo_db.Country')),
                ('geocoordinate', models.ForeignKey(null=True, to='django_geo_db.GeoCoordinate', help_text='This is a very specific location.', blank=True)),
            ],
        ),
        migrations.CreateModel(
            name='State',
            fields=[
                ('state_id', models.AutoField(serialize=False, primary_key=True)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('abbreviation', models.CharField(max_length=2, unique=True)),
                ('generated_name', models.CharField(null=True, max_length=50, blank=True)),
                ('country', models.ForeignKey(to='django_geo_db.Country')),
                ('geocoordinate', models.ForeignKey(to='django_geo_db.GeoCoordinate')),
            ],
        ),
        migrations.CreateModel(
            name='UserGeographySettings',
            fields=[
                ('user_geography_settings_id', models.AutoField(serialize=False, primary_key=True)),
                ('location', models.ForeignKey(to='django_geo_db.Location')),
                ('user', models.OneToOneField(to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='UserLocation',
            fields=[
                ('user_location_id', models.AutoField(serialize=False, primary_key=True)),
                ('last_used', models.DateTimeField(default=datetime.datetime.now, blank=True)),
                ('location', models.ForeignKey(to='django_geo_db.Location')),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ('last_used',),
            },
        ),
        migrations.CreateModel(
            name='Zipcode',
            fields=[
                ('zipcode_id', models.AutoField(serialize=False, primary_key=True)),
                ('zipcode', IntegerRangeField(unique=True)),
                ('timezone', models.IntegerField()),
                ('generated_name', models.CharField(null=True, max_length=50, blank=True)),
                ('city', models.ForeignKey(to='django_geo_db.City')),
                ('geocoordinate', models.ForeignKey(to='django_geo_db.GeoCoordinate')),
            ],
        ),
        migrations.AddField(
            model_name='location',
            name='state',
            field=models.ForeignKey(null=True, to='django_geo_db.State', blank=True),
        ),
        migrations.AddField(
            model_name='location',
            name='zipcode',
            field=models.ForeignKey(null=True, to='django_geo_db.Zipcode', blank=True),
        ),
        migrations.AlterUniqueTogether(
            name='geocoordinate',
            unique_together=set([('lat', 'lon')]),
        ),
        migrations.AddField(
            model_name='country',
            name='geocoordinate',
            field=models.ForeignKey(to='django_geo_db.GeoCoordinate'),
        ),
        migrations.AddField(
            model_name='city',
            name='geocoordinate',
            field=models.ForeignKey(to='django_geo_db.GeoCoordinate'),
        ),
        migrations.AddField(
            model_name='city',
            name='state',
            field=models.ForeignKey(to='django_geo_db.State'),
        ),
        migrations.AlterUniqueTogether(
            name='zipcode',
            unique_together=set([('zipcode', 'geocoordinate')]),
        ),
        migrations.AlterUniqueTogether(
            name='userlocation',
            unique_together=set([('user', 'location')]),
        ),
        migrations.AlterUniqueTogether(
            name='state',
            unique_together=set([('country', 'name')]),
        ),
        migrations.AlterUniqueTogether(
            name='location',
            unique_together=set([('country', 'state', 'city', 'zipcode', 'geocoordinate')]),
        ),
        migrations.AlterUniqueTogether(
            name='city',
            unique_together=set([('state', 'name')]),
        ),
    ]



from django_geo_db import views
from django.conf.urls import url

model_urls = [
    url(r'^api/v1/geocoordinate/(?P<pk>\d+)/$', views.GeoCoordinateDetails.as_view(), name='geocoordinate-detail'),
    url(r'^api/v1/continent/(?P<pk>\d+)/$', views.ContinentDetails.as_view(), name='continent-detail'),
    url(r'^api/v1/country/(?P<pk>\d+)/$', views.CountryDetails.as_view(), name='country-detail'),
    url(r'^api/v1/state/(?P<pk>\d+)/$', views.StateDetails.as_view(), name='state-detail'),
    url(r'^api/v1/city/(?P<pk>\d+)/$', views.CityDetails.as_view(), name='city-detail'),
    url(r'^api/v1/zipcode/(?P<pk>\d+)/$', views.ZipcodeDetails.as_view(), name='zipcode-detail'),
    url(r'^api/v1/location/(?P<pk>[0-9]+)/$', views.LocationDetail.as_view(), name='location-detail'),

    url(r'^api/v1/geocoordinate/', views.GeoCoordinateList.as_view(), name='geocoordinate-list'),
    url(r'^api/v1/continent/', views.ContinentList.as_view(), name='continent-list'),
    url(r'^api/v1/country/', views.CountryList.as_view(), name='country-list'),
    url(r'^api/v1/city/', views.CityList.as_view(), name='city-list'),
    url(r'^api/v1/state/', views.StateList.as_view(), name='state-list'),
    url(r'^api/v1/zipcode/', views.ZipcodeList.as_view(), name='zipcode-list'),
    url(r'^api/v1/location/', views.LocationList.as_view(), name='location-list'),
]




from django.core.management.base import BaseCommand
from django_geo_db.models import Location, Zipcode, City, State, GeoCoordinate
from django.db import IntegrityError, transaction

@transaction.atomic
class Command(BaseCommand):
    help = "Sets up all of the generated names of geo objects"

    def __generate(self, modelType):
        models = modelType.objects.all()
        count = models.count()
        print('Generating {0}: {1}'.format(modelType.__name__, count))
        buffer = []
        for index in range(count):
            l = models[index]
            buffer.append(l)
            if len(buffer) > 1000:
                try:
                    with transaction.atomic():
                        for l in buffer:
                            l.save()
                    print('Saved {0}'.format(index))
                except IntegrityError as e:
                    print('Exception occurred while committing.')
                    raise e
                buffer = []

    def handle(self, *args, **options):
        self.__generate(Location)
        self.__generate(Zipcode)
        self.__generate(City)
        self.__generate(State)
        self.__generate(GeoCoordinate)





from django.conf import settings
from django_geo_db.services import GEO_DAL


def google_maps_api_key(request):
    return {'GOOGLE_MAPS_API_KEY': settings.GOOGLE_MAPS_API_KEY}


def google_maps_settings(request):
    return {'GM_SETTINGS': settings.GM_SETTINGS}




from decimal import Decimal

def get_lat_lon_from_string(latLonString):
    lat,lon = latLonString.split(' ')
    lat = get_standardized_coordinate(lat)
    lon = get_standardized_coordinate(lon)
    return (Decimal(lat), Decimal(lon))

def get_standardized_coordinate(latOrLon):
    objInt, objFrac = str(latOrLon).split('.', 1)
    objFrac = str(objFrac)[0:5]
    return Decimal('{0}.{1}'.format(objInt, objFrac))



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-18 15:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_geo_db', '0002_location_is_private'),
    ]

    operations = [
        migrations.AddField(
            model_name='userlocation',
            name='user_created',
            field=models.BooleanField(default=True),
        ),
    ]



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django_geo_db',
    version='0.1.4',
    packages=find_packages(),
    include_package_data=True,
    license='GPLv3',  
    description='A Django app for enabling geo capabilities.',
    long_description=README,
    author='Mike Cornwell',
    install_requires=['django', 'djangorestframework', 'django_autocomplete_light', 'six'],
    author_email='mike@mikecornwell.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)




from django_geo_db import autocomplete_views
from django.conf.urls import url

autocomplete_urls = [
    url('^autocomplete/named-location/$',
        views.NamedLocationAutocomplete.as_view(),
        name='named-location-autocomplete'),
    url('^autocomplete/location/$',
        views.LocationAutocomplete.as_view(),
        name='location-autocomplete'),
    url('^autocomplete/public-locations/$',
        views.PublicLocationsAutocomplete.as_view(),
        name='public-locations-autocomplete'),
    url('^autocomplete/zipcode/$',
        views.ZipcodeAutocomplete.as_view(),
        name='zipcode-autocomplete'),
    url('^autocomplete/city/$',
        views.CityAutocomplete.as_view(),
        name='city-autocomplete'),
    url('^autocomplete/geocoordinate/$',
        views.GeoCoordinateAutocomplete.as_view(),
        name='geocoordinate-autocomplete'),
]



from django.contrib import admin

from django_geo_db.models import Location, City, Continent, Country, \
    State, GeoCoordinate, UserLocation
from django_geo_db.forms import UserLocationForm, LocationForm, CityForm, GeocoordinateForm


class UserLocationAdmin(admin.ModelAdmin):
    form = UserLocationForm

class LocationAdmin(admin.ModelAdmin):
    form = LocationForm

class CityAdmin(admin.ModelAdmin):
    form = CityForm

class GeocoordinateAdmin(admin.ModelAdmin):
    form = GeocoordinateForm




admin.site.register(City, CityAdmin)
admin.site.register(Continent)
admin.site.register(Country)
admin.site.register(State)
admin.site.register(GeoCoordinate, GeocoordinateAdmin)
admin.site.register(UserLocation, UserLocationAdmin)
admin.site.register(Location, LocationAdmin)





from decimal import Decimal
from datetime import datetime
from django.db import models
from django.contrib.auth.models import User
from django_geo_db.managers import LocationManager, GeoCoordinateManager
from django_geo_db.utilities import get_standardized_coordinate


class IntegerRangeField(models.IntegerField):
    def __init__(self, verbose_name=None, name=None, min_value=None, max_value=None, **kwargs):
        self.min_value, self.max_value = min_value, max_value
        models.IntegerField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'min_value': self.min_value, 'max_value':self.max_value}
        defaults.update(kwargs)
        return super(IntegerRangeField, self).formfield(**defaults)


class GeoCoordinate(models.Model):
    objects = GeoCoordinateManager()
    geocoordinate_id = models.AutoField(primary_key=True)
    lat = models.DecimalField(max_digits=7, decimal_places=5)
    lon = models.DecimalField(max_digits=8, decimal_places=5)
    generated_name = models.CharField(max_length=50, blank=True, null=True)

    def __str__(self):
        return self.generated_name or self.__get_generated_name()

    def save(self, *args, **kwargs):
        self.__standardize_fractionals()
        self.generated_name = self.__get_generated_name()
        super(GeoCoordinate, self).save(*args, **kwargs)

    def __get_generated_name(self):
        return '{0} {1}'.format(self.lat, self.lon)

    def __standardize_fractionals(self):
        self.lat = get_standardized_coordinate(self.lat)
        self.lon = get_standardized_coordinate(self.lon)

    class Meta:
        unique_together = ('lat', 'lon')


class Continent(models.Model):
    continent_id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=50, unique=True)

    def __str__(self):
        return self.name


class Country(models.Model):
    country_id = models.AutoField(primary_key=True)
    continent = models.ForeignKey(Continent)
    name = models.CharField(max_length=50, unique=True)
    abbreviation = models.CharField(max_length=2, unique=True)
    geocoordinate = models.ForeignKey(GeoCoordinate)

    def __str__(self):
        return self.name


class State(models.Model):
    state_id = models.AutoField(primary_key=True)
    country = models.ForeignKey(Country)
    name = models.CharField(max_length=50, unique=True)
    abbreviation = models.CharField(max_length=2, unique=True)
    geocoordinate = models.ForeignKey(GeoCoordinate)
    generated_name = models.CharField(max_length=50, blank=True, null=True)

    def save(self, *args, **kwargs):
        self.generated_name = self.__get_generated_name()
        return super(State, self).save(*args, **kwargs)

    def __str__(self):
        return self.generated_name or self.__get_generated_name()

    def __get_generated_name(self):
        return str('{0}, {1}'.format(self.name, self.country.abbreviation))

    class Meta:
        unique_together = (('country', 'name'),)


class City(models.Model):
    city_id = models.AutoField(primary_key=True)
    state = models.ForeignKey(State)
    name = models.CharField(max_length=50)
    geocoordinate = models.ForeignKey(GeoCoordinate)
    generated_name = models.CharField(max_length=50, blank=True, null=True)

    def save(self, *args, **kwargs):
        self.generated_name = self.__get_generated_name()
        return super(City, self).save(*args, **kwargs)

    class Meta:
        unique_together = (('state', 'name'),)

    def __str__(self):
        return self.generated_name or self.__get_generated_name()

    def __get_generated_name(self):
        return str('{0}, {1}'.format(self.name, self.state.name))


class Zipcode(models.Model):
    zipcode_id = models.AutoField(primary_key=True)
    city = models.ForeignKey(City)
    zipcode = IntegerRangeField(unique=True, min_value=1, max_value=99999)
    geocoordinate = models.ForeignKey(GeoCoordinate)
    timezone = models.IntegerField()
    generated_name = models.CharField(max_length=50, blank=True, null=True)

    class Meta:
        unique_together = ('zipcode', 'geocoordinate')

    def save(self, *args, **kwargs):
        self.generated_name = self.__get_generated_name()
        return super(Zipcode, self).save(*args, **kwargs)

    def __str__(self):
        return self.generated_name or self.__get_generated_name()

    def __get_generated_name(self):
        return str('{0}, {1} {2}'.format(self.city.name, self.city.state.name, self.zipcode))


class Location(models.Model):
    """
    The working horse of locational data.
    This is the object that should be a foreign key to MOST geolocated objects.
    It is a cascading of Locational information that gets more detailed depending on
    what level of granularity is desired, the minimum being a country.
    """
    objects = LocationManager()
    location_id = models.AutoField(primary_key=True)
    country = models.ForeignKey(Country)
    state = models.ForeignKey(State, blank=True, null=True)
    city = models.ForeignKey(City, blank=True, null=True)
    zipcode = models.ForeignKey(Zipcode, blank=True, null=True)
    geocoordinate = models.ForeignKey(GeoCoordinate, blank=True, null=True, help_text='This is a very specific location.')
    name = models.CharField(max_length=30, blank=True, null=True)
    is_private = models.BooleanField(default=False, help_text='Is this a location that should not be available to be seen publically?')
    generated_name = models.CharField(max_length=50, blank=True, null=True)

    def __str__(self):
        return self.generated_name or self.__get_generated_name()

    def get_geocoordinate(self):
        """
        Gets the most specific geocoordinate for this Location.
        :return:
        """
        if self.geocoordinate:
            return self.geocoordinate
        obj = self.zipcode
        if not obj:
            obj = self.city
        if not obj:
            obj = self.state
        if not obj:
            obj = self.country
        return obj.geocoordinate

    def lat(self):
        return self.get_geocoordinate().lat

    def lon(self):
        return self.get_geocoordinate().lon

    def lat_lon(self):
        return str(self.get_geocoordinate())

    def save(self, *args, **kwargs):
        self.__set_cascading_details()
        self.__set_generated_name()
        self.__validate_interconnection_details()
        return super(Location, self).save(*args, **kwargs)

    def __set_cascading_details(self):
        if self.zipcode and not self.city:
            self.city = self.zipcode.city
        if self.city and not self.state:
            self.country = self.city.state
        if self.state and not self.country:
            self.country = self.state.country

    def __validate_interconnection_details(self):
        if self.state and self.state.country != self.country:
            raise Exception("The state's country does not match the selected country.")
        if self.city and self.city.state != self.state:
            raise Exception("City's state does not match the selected state.")
        if self.zipcode and self.zipcode.city != self.city:
            raise Exception("Zipcode's city does not match the selected city.")
        if self.name and not self.geocoordinate:
            raise Exception('Names must have a geocoordinate.')
        if not self.name and self.geocoordinate:
            raise Exception('Geocoordinates must be named.')

    def __set_generated_name(self):
        self.generated_name = self.__get_generated_name()

    def __get_generated_name(self):
        value = ''
        if self.name:
            value = '{0} ({1})'.format(self.name, self.geocoordinate)
        elif self.zipcode:
            value = '{0}, {1} {2}'.format(self.city.name, self.state.abbreviation, self.zipcode.zipcode)
        elif self.city:
            value = '{0}, {1}'.format(self.city.name, self.state.abbreviation)
        elif self.state:
            value = '{0}, {1}'.format(self.state.name, self.country.name)
        else:
            value = '{0}'.format(self.country.name)
        return value

    class Meta:
        unique_together = ('country', 'state', 'city', 'zipcode', 'geocoordinate')


class UserLocation(models.Model):
    """
    A location that a user has used in the past.
    """
    user_location_id = models.AutoField(primary_key=True)
    user = models.ForeignKey(User)
    location = models.ForeignKey(Location)
    last_used = models.DateTimeField(default=datetime.now, blank=True)
    user_created = models.BooleanField(default=True)

    def save(self, *args, **kwargs):
        self.last_used = datetime.now()
        return super(UserLocation, self).save(args, kwargs)

    def __str__(self):
        return self.location.name

    class Meta:
        ordering = ('last_used',)
        unique_together = ('user', 'location')



# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2017-05-18 17:26
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django_geo_db', '0003_userlocation_user_created'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='usergeographysettings',
            name='location',
        ),
        migrations.RemoveField(
            model_name='usergeographysettings',
            name='user',
        ),
        migrations.DeleteModel(
            name='UserGeographySettings',
        ),
    ]



from django import forms
from django.conf import settings
from django.template.loader import render_to_string

class GeocoordinateWidget(forms.TextInput):
    template_name = 'geocoordinate.html'
    css = {
            'all': ('styles.css',)
        }

    def render(self, name, value, attrs=None):
        context = {
            'name': name,
            'value': value,
            'GM_SETTINGS': settings.GM_SETTINGS,
            'GOOGLE_MAPS_API_KEY': settings.GOOGLE_MAPS_API_KEY
        }
        return render_to_string(self.template_name, context)
